@@ -34,20 +34,27 @@ def mkf_input(snts, docs, tokenizer, sntdic:dict={},diffmerge:bool=False):
 
 def todsk(essay_or_snts:str="She has ready. It are ok.", asdsk:bool=True, dskhost:str='gpu120.wrask.com:7095'  
 		, debug:bool= False
+		, redis_r	= None # xadd, redis.r		= redis.Redis(host=rhost, port=rport, db=rdb, decode_responses=True) 
+		, redis_bs	= None # bs cache, bytes, redis.Redis(host=rhost, port=rport, db=rdb, decode_responses=False) 
 		, gec_func	= lambda snts: requests.post(f"http://gpu120.wrask.com:8180/redis/getgecs", json=snts).json () 
-		, nlp_func	= lambda snt: spacy.nlp(snt) ): 
+		, nlp_func	= lambda snt: spacy.nlp(snt)
+		, polish_f	= lambda doc, dsk, idx: None): # extending polishment terms
 	''' online version gec, no pipe imported, 2022.5.30 '''
 	try:
 		tims	= [ ("start", time.time(), 0)] # tag, tm, delta 
 		snts	= json.loads(essay_or_snts) if essay_or_snts.startswith("[") else en.sntbr(essay_or_snts)
+		if redis_r: [redis_r.xadd('xsntbytes', {'snt':snt}) for snt in snts] # notify spacy snt parser
+
 		sntdic	= gec_func(snts) #{'She has ready.': 'She is ready.', 'It are ok.': 'It is ok.'}
 		if debug : tims.append( ("gec", time.time(), round(time.time() - tims[-1][1],2))  )
-		docs	= [ nlp_func(snt) for snt in snts ] 
+		docs	= [ en.getdoc(snt, redis_bs.get(f"bytes:{snt}") ) if redis_bs else nlp_func(snt) for snt in snts ] 
 		if debug : tims.append( ("nlp", time.time(), round(time.time() - tims[-1][1],2))  )
 		input	= mkf_input(snts, docs, spacy.nlp.tokenizer, sntdic)
 		dsk		= requests.post(f"http://{dskhost}/parser", data={"q":json.dumps({"snts":input, "rid":"10"} if asdsk else input).encode("utf-8")}).json()
 		if debug : tims.append( ("dsk", time.time(), round(time.time() - tims[-1][1], 2))  )
 		if debug and isinstance(dsk, dict) and 'info' in dsk : dsk['info']['tim'] = tims #[('start', 1653811771.030599, 0), ('gec', 1653811776.2294545, 5.2), ('nlp', 1653811776.2439919, 0.01), ('dsk', 1653811776.275237, 0.03)]
+
+		[polish_f(doc, dsk, idx ) for idx, doc in enumerate(docs) ]
 		return dsk  #docker run -d --restart=always --name dsk17095 -v /data/dct:/dct -p 7095:7095 wrask/gec:dsk8 java -Xmx4096m -jar pigai_engine8.jar --database-no-encrypt --server-addr dsk.wrask.com --server-port 7095  --database-type sqlite --sqlite-file dct/sqlite/pigai_spss.sqlite3 --thread-num 2 --gec-snts-address http://wrask.com:33000/gec/essay_or_snts
 	except Exception as ex: 
 		print(">>todsk Ex:", ex, "\t|", essay_or_snts)
@@ -56,7 +63,7 @@ def todsk(essay_or_snts:str="She has ready. It are ok.", asdsk:bool=True, dskhos
 
 def localgec_todsk(essay_or_snts:str="She has ready. It are ok.", device:int=-1, asdsk:bool=True, dskhost:str='172.17.0.1:7095', debug:bool=False): 
 	''' essay -> dsk, with local gec model support, |  2022.5.28 '''
-	import pipe #pip install torch transformers|
+	import pipe #pip install torch transformers| cp model
 	return todsk(essay_or_snts, asdsk=asdsk, dskhost=dskhost, debug=debug, gec_func = lambda snts: pipe.gecsnts(snts, device=device))
 
 def init_dskmkf_table(cursor):
@@ -75,7 +82,7 @@ def init_dskmkf_table(cursor):
   UNIQUE KEY `eidv` (`eid`,`ver`),
   KEY `rid` (`rid`),
   KEY `uid` (`uid`)
-) ENGINE=MyISAM DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci ''')
+) ENGINE=Innodb DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci ''')
 
 	cursor.execute('''CREATE TABLE if not exists `mkf` (
   `sntmd5` char(32) COLLATE utf8mb4_unicode_ci NOT NULL,
@@ -85,11 +92,10 @@ def init_dskmkf_table(cursor):
   `chunk` json DEFAULT NULL,
   `meta` json DEFAULT NULL,
   `feedback` json DEFAULT NULL,
-   bs longblob, 
   `tm` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
   PRIMARY KEY (`sntmd5`),
   FULLTEXT KEY `sntkps` (`snt`,`kps`)
-) ENGINE=MyISAM DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci''')
+) ENGINE=Innodb DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci''')
 
 import hashlib
 md5snt	= lambda text: hashlib.md5(text.encode("utf-8")).hexdigest()
@@ -121,12 +127,22 @@ def submit_dskmkf(dsk, cursor):
 			kps = [ f"{t.pos_}_{t.lemma_}" for t in doc] + [ f"{t.tag_}_{t.lemma_}" for t in doc] + [ f"{t.dep_}_{t.head.pos_}_{t.pos_}_{t.head.lemma_}_{t.lemma_}" for t in doc if t.pos_ not in ('PRON','PUNCT') and t.dep_ in ('dobj','nsubj','advmod','acomp','amod','compound','xcomp','ccomp')]
 			[ kps.append( ar.get('cate','').replace('.','_')) for ar in fds if ar.get('cate','')] # e_prep.wrong -> e_prep_wrong
 
-			cursor.execute("insert ignore into mkf(sntmd5, snt, kps, tok, chunk, meta, feedback,bs) values(%s,%s,%s,%s,%s,%s,%s,%s)", (sntmd5, snt, ' '.join(kps),
-			json.dumps(doc_tok(doc)), json.dumps(doc_chunk(doc)), json.dumps(meta), json.dumps(fds), spacy.tobs(doc)  ) 	)
+			cursor.execute("insert ignore into mkf(sntmd5, snt, kps, tok, chunk, meta, feedback) values(%s,%s,%s,%s,%s,%s,%s)", (sntmd5, snt, ' '.join(kps),
+			json.dumps(doc_tok(doc)), json.dumps(doc_chunk(doc)), json.dumps(meta), json.dumps(fds)  ) 	) #, spacy.tobs(doc)
+
+def polish_func(doc, dsk, idx): 
+	''' '''
+	lookup	= {"overcome difficulty/dobj_VERB_NOUN": {"conquer difficulty":1, 'short_msg': 'conquer, surmount'}}
+	mkf		= dsk['snt'][idx] 
+	for t in doc: 
+		term = f"{t.head.lemma_} {t.lemma_}/{t.dep_}_{t.head.pos_}_{t.pos_}"
+		if term in lookup: 
+			mkf['feedback'].update({ f"{term}@r_polish:{t.i}": dict(lookup[term], **{"ibeg":t.i}) } )
 
 if __name__ == '__main__':
-	res = todsk(dskhost="gpu120.wrask.com:7095", debug=True)
-	print ( res ) 
+	res = todsk("I overcame the difficulties.", polish_f = polish_func, debug=True)
+	#{'feedback': 'overcome difficulty/dobj_VERB_NOUN@r_polish:3': {'conquer difficulty': 1, 'short_msg': 'conquer, surmount', 'ibeg': 3}}, 'meta': {'pid': 0, 'ske': ['n_v_n'], 'para_id': 0, 'sid': 0, 'tc': 5, 'sub_cnt': 1, 'pos_rewrite': '[^/^, I/PRP, overcame/VBD, the/DT, difficulties/NNS, ./.]', 'pred_lemma': 'overcome', 'postag': '^_^_^ I_prp_prp_no_n_sb_I overcame_vbd_pastten_v_overcome the_dt_n2_the difficulties_nns_n_difficulty ._._.', 'snt': 'I overcame the difficulties.', 'lex_list': 'I overcame the difficulties .', 'vpat': ['overcome _n'], 'tense': ''}}
+	print ( res['snt'][0] ) 
 	print ( res['info']['tim'])
 
 # cp __init__.py /home/ubuntu/.local/lib/python3.8/site-packages/dsk
\ No newline at end of file
