@@ -82,6 +82,12 @@ def rgetdoc(rbs, snt, prefix="bs:", ttl:int=37200):
 def getdoc(snt, bs):  # execute("GET", f"bytes:{snt}") 
 	return Doc(spacy.nlp.vocab).from_bytes(bs) if bs else spacy.nlp(snt) 
 
+def phrase_matcher(name:str='pp', patterns:list=[[{'POS': 'ADP'},{"POS": {"IN": ["DET","NUM","ADJ",'PUNCT','CONJ']}, "OP": "*"},{"POS": {"IN": ["NOUN","PART"]}, "OP": "+"}]]):
+	''' for name, ibeg,iend in matcher(doc) : print(spacy.nlp.vocab[name].text, doc[ibeg:iend].text) '''
+	matcher = Matcher(spacy.nlp.vocab)
+	matcher.add(name, patterns, greedy ='LONGEST')
+	return matcher
+
 def sntbr(essay, trim:bool=False, with_pid:bool=False): 
 	''' added 2022.5.28 '''
 	from spacy.lang import en
