@@ -112,6 +112,7 @@ def verbnet(doc):
 simple_sent		= lambda doc: len([t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) <= 0 
 complex_sent	= lambda doc: len([t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) > 0
 compound_sent	= lambda doc: len([t for t in doc if t.dep_ == 'conj' and t.head.dep_ == 'ROOT']) > 0  # S.conj 
+pred_offset		= lambda doc:  (ar := [ t.i for t in doc if t.dep_ == "ROOT"], offset := ar[0] if len(ar) > 0 else 0,  offset/( len(doc) + 0.1) )[-1]
 
 def stype(doc): 
 	doc.user_data["stype"] = {"type": "simple_sent" if len([t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) <= 0 else "complex_sent" }
@@ -138,7 +139,7 @@ def attach(doc):
 
 	# last to be called , added 2022.3.24,   in ES:  skenp:postag , to searh "_^ _IN _NP ," , in paper ratio
 	merge_np(doc) 
-	doc.user_data["skenp"] = {"type":"skenp", "chunk": "_^ " + " ".join([ "_NP" if t.ent_type_ == 'NP' else f"{t.text}_{t.lemma_}_{t.pos_}_{t.tag_}" for t in doc]) + " _$"}
+	doc.user_data["skenp"] = {"type":"skenp", "snt": doc.text, "skenp": "_^ " + " ".join([ "_NP" if t.ent_type_ == 'NP' else f"{t.text}_{t.lemma_}_{t.pos_}_{t.tag_}" for t in doc]) + " _$"}
 
 if __name__ == "__main__":  
 	doc = spacy.nlp("Considering the possibility, it was ok, and she is ok.")
@@ -193,4 +194,7 @@ def submit_verbnet(sid, doc, ssv): 	#ssv = defaultdict(dict)
  ['S.nsubjpass', 3],
  ['S.appos', 1]]
 
+	doc.user_data["snt"] = {'type':'snt', 'snt':doc.text, 	'pred_offset': pred_offset(doc), 
+				'postag':'_^ ' + ' '.join([f"{t.text}_{t.lemma_}_{t.pos_}_{t.tag_}" if t.text == t.text.lower() else f"{t.text}_{t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}" for t in doc]) + ' _$',
+			   'tc': len(doc)} #'sid': rowid, 'src': rowid,
 '''
\ No newline at end of file
