@@ -1,4 +1,4 @@
-# 2022.5.28  
+# 2022.6.4
 import re, math #torch
 from transformers import pipeline #None of PyTorch, TensorFlow >= 2.0, or Flax have been found. Models won't be available and only tokenizers, configuration and file/data utilities can be used.
 
@@ -27,5 +27,31 @@ def gecsnts(snts:list=["She has ready.","It are ok."],  max_length:int=128,  do_
 		offset = offset + batch_size
 	return dic
 
+## wrapper web api for uvicorn
+
+def redis_getgecs(snts:list=["She has ready.","It are ok."], xname:str='xsnts', host:str="172.17.0.1", port:int=6379, db:int=0, timeout:int=5): # put into the ufw white ip list 
+	''' snts -> gecs '''
+	import redis
+	try:
+		if not hasattr(redis, 'r'): redis.r = redis.Redis(host=host,port=port, db=db, decode_responses=True)
+		id  = redis.r.xadd(xname, {"snts":json.dumps(snts)})
+		res	= redis.r.blpop([f"suc:{id}",f"err:{id}"], timeout=timeout)
+		return {} if res is None else json.loads(res[1])
+	except Exception as e:
+		print ("redis_getgecs ex:", e, snts) 
+		return {}
+
+def getgecs_with_backoff(snts:list=["She has ready.","It are ok."] , local:bool= False, local_as_backoff:bool=True
+			, xname:str='xsnts', host:str="172.17.0.1", port:int=6379, db:int=0, timeout:int=5
+			, max_length:int=128,  do_sample:bool=False, batch_size:int=64, unchanged_ratio:float=0.45, len_ratio:float=0.5, model:str="/grammar_error_correcter_v1", device:int=-1):
+	''' main gec api, 1. redis_gec  2. when failed , call local_gec, 2022.6.3 '''
+	if not snts: return {} 
+	if local: return gecsnts(snts, max_length=max_length,do_sample=do_sample, batch_size =batch_size, unchanged_ratio=unchanged_ratio, len_ratio = len_ratio, model =model, device=device)
+
+	sntdic = redis_getgecs(snts, xname=xname, host=host, port=port, db=db, timeout=timeout ) #
+	if not sntdic and local_as_backoff:  # failed , call the local version as backoff
+		sntdic = gecsnts(snts, max_length=max_length,do_sample=do_sample, batch_size =batch_size, unchanged_ratio=unchanged_ratio, len_ratio = len_ratio, model =model, device=device)
+	return sntdic
+
 if __name__ == "__main__":   #{'She has ready.': 'She is ready.', 'It are ok.': 'It is ok.'}
 	print ( gecsnts() )
\ No newline at end of file
